@@ -3,6 +3,7 @@
 
 // defines like IS_DIGIT, etc'
 #include "r_util/r_str_util.h"
+#include "r_userconf.h"
 
 // TODO: fix this to make it crosscompile-friendly: R_SYS_OSTYPE ?
 /* operating system */
@@ -42,10 +43,16 @@
 #define LIBC_HAVE_SYSTEM 1
 #endif
 
-#if APPLE_SDK_APPLETVOS || APPLE_SDK_WATCHOS || APPLE_SDK_APPLETVSIMULATOR || APPLE_SDK_WATCHSIMULATOR
-#define LIBC_HAVE_FORK 0
+#if APPLE_SDK_IPHONEOS || APPLE_SDK_APPLETVOS || APPLE_SDK_WATCHOS || APPLE_SDK_APPLETVSIMULATOR || APPLE_SDK_WATCHSIMULATOR
+#define LIBC_HAVE_PTRACE 0
 #else
+#define LIBC_HAVE_PTRACE 1
+#endif
+
+#if HAVE_FORK
 #define LIBC_HAVE_FORK 1
+#else
+#define LIBC_HAVE_FORK 0
 #endif
 
 #if defined(__OpenBSD__)
@@ -81,6 +88,12 @@
 #define MINGW32 1
 #endif
 
+#ifdef _MSC_VER
+  #define strcasecmp stricmp
+  #define strncasecmp strnicmp
+  #define __WINDOWS__ 1
+#endif
+
 #if defined(EMSCRIPTEN) || defined(__linux__) || defined(__APPLE__) || defined(__GNU__) || defined(__ANDROID__) || defined(__QNX__) || defined(__sun)
   #define __BSD__ 0
   #define __UNIX__ 1
@@ -89,18 +102,25 @@
   #define __BSD__ 1
   #define __UNIX__ 1
 #endif
-#if __WINDOWS__ || _WIN32 || __CYGWIN__ || MINGW32
-  #define __addr_t_defined
-  #include <windows.h>
-#endif
 #if __WINDOWS__ || _WIN32 || MINGW32 && !(__MINGW64__ || __CYGWIN__)
-  #include <winsock.h>
+  #ifdef _MSC_VER
+  /* Must be included before windows.h */
+  #include <winsock2.h>
+  #define WIN32_LEAN_AND_MEAN
+  #else
+  /* Deprecated */
+  //#include <winsock.h>
+  #endif
   typedef int socklen_t;
   #undef USE_SOCKETS
   #define __WINDOWS__ 1
   #undef __UNIX__
   #undef __BSD__
 #endif
+#if __WINDOWS__ || _WIN32 || __CYGWIN__ || MINGW32
+  #define __addr_t_defined
+  #include <windows.h>
+#endif
 
 #if defined(__APPLE__) && (__arm__ || __arm64__ || __aarch64__)
 #define TARGET_OS_IPHONE 1
@@ -148,7 +168,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <sys/time.h>
 #include <fcntl.h> /* for O_RDONLY */
 #include <r_endian.h> /* needs size_t */
 
@@ -156,11 +175,6 @@
 extern "C" {
 #endif
 
-#define R_LIB_VERSION_HEADER(x) \
-const char *x##_version()
-#define R_LIB_VERSION(x) \
-const char *x##_version () { return "" R2_GITTAP; }
-
 #define TODO(x) eprintf(__func__"  " x)
 
 // TODO: FS or R_SYS_DIR ??
@@ -223,11 +237,18 @@ typedef void (*PrintfCallback)(const char *str, ...);
 #else
   #if defined(__GNUC__) && __GNUC__ >= 4
     #define R_API __attribute__((visibility("default")))
+  #elif defined(_MSC_VER)
+    #define R_API __declspec(dllexport)
   #else
     #define R_API
   #endif
 #endif
 
+#define R_LIB_VERSION_HEADER(x) \
+R_API const char *x##_version(void)
+#define R_LIB_VERSION(x) \
+R_API const char *x##_version () { return "" R2_GITTAP; }
+
 #define BITS2BYTES(x) (((x)/8)+(((x)%8)?1:0))
 #define ZERO_FILL(x) memset (&x, 0, sizeof (x))
 #define R_NEWS0(x,y) (x*)calloc(y,sizeof(x))
@@ -235,6 +256,7 @@ typedef void (*PrintfCallback)(const char *str, ...);
 #define R_NEW0(x) (x*)calloc(1,sizeof(x))
 #define R_NEW(x) (x*)malloc(sizeof(x))
 #define R_NEWCOPY(x,y) (x*)r_new_copy(sizeof(x), y)
+
 static inline void *r_new_copy(int size, void *data) {
 	void *a = malloc(size);
 	if (a) {
@@ -250,13 +272,16 @@ static inline void *r_new_copy(int size, void *data) {
 
 #define R_PTR_ALIGN(v,t) \
 	((char *)(((size_t)(v) ) \
-	& ~(t - 1))) 
+	& ~(t - 1)))
 #define R_PTR_ALIGN_NEXT(v,t) \
 	((char *)(((size_t)(v) + (t - 1)) \
-	& ~(t - 1))) 
+	& ~(t - 1)))
 
 #define R_BIT_SET(x,y) (((ut8*)x)[y>>4] |= (1<<(y&0xf)))
 #define R_BIT_UNSET(x,y) (((ut8*)x)[y>>4] &= ~(1<<(y&0xf)))
+#define R_BIT_SWAP(x, y) ( R_BIT_CHK (x, y) ? \
+		R_BIT_UNSET (x, y): R_BIT_SET (x, y))
+
 //#define R_BIT_CHK(x,y) ((((const ut8*)x)[y>>4] & (1<<(y&0xf))))
 #define R_BIT_CHK(x,y) (*(x) & (1<<(y)))
 
@@ -278,9 +303,7 @@ static inline void *r_new_copy(int size, void *data) {
 #define perror(x) _perror(x,__FILE__,__LINE__,__func__)
 #define r_sys_perror(x) _perror(x,__FILE__,__LINE__,__func__)
 #else
-
-#define r_sys_perror(x) perror(x);
-
+#define r_sys_perror(x) r_sys_perror_str(x);
 #endif
 
 #if __UNIX__
@@ -288,11 +311,12 @@ static inline void *r_new_copy(int size, void *data) {
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
-#endif
 #include <unistd.h>
+#include <sys/time.h>
+#endif
 
 #ifndef HAVE_EPRINTF
-#define eprintf(x,y...) fprintf(stderr,x,##y)
+#define eprintf(...) fprintf(stderr,__VA_ARGS__)
 #define eprint(x) fprintf(stderr,"%s\n",x)
 #define HAVE_EPRINTF 1
 #endif
@@ -301,6 +325,7 @@ static inline void *r_new_copy(int size, void *data) {
 #define typeof(arg) __typeof__(arg)
 #endif
 
+#undef r_offsetof
 #define r_offsetof(type, member) ((unsigned long) &((type*)0)->member)
 
 #define R_BETWEEN(x,y,z) (((y)>=(x)) && ((y)<=(z)))
@@ -372,34 +397,65 @@ static inline void *r_new_copy(int size, void *data) {
 #if __i386__
 #define R_SYS_ARCH "x86"
 #define R_SYS_BITS R_SYS_BITS_32
+#define R_SYS_ENDIAN 0
 #elif __x86_64__
 #define R_SYS_ARCH "x86"
 #define R_SYS_BITS (R_SYS_BITS_32 | R_SYS_BITS_64)
-#elif __POWERPC__
-#define R_SYS_ARCH "ppc"
-#define R_SYS_BITS R_SYS_BITS_32
+#define R_SYS_ENDIAN 0
+#elif __powerpc__
+# define R_SYS_ARCH "ppc"
+# ifdef __powerpc64__
+#  define R_SYS_BITS (R_SYS_BITS_32 | R_SYS_BITS_64)
+# else
+#  define R_SYS_BITS R_SYS_BITS_32
+# endif
+# if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+#  define R_SYS_ENDIAN 0
+# else
+#  define R_SYS_ENDIAN 1
+# endif
 #elif __arm__
 #define R_SYS_ARCH "arm"
 #define R_SYS_BITS R_SYS_BITS_32
+#define R_SYS_ENDIAN 0
 #elif __arm64__ || __aarch64__
 #define R_SYS_ARCH "arm"
 #define R_SYS_BITS (R_SYS_BITS_32 | R_SYS_BITS_64)
+#define R_SYS_ENDIAN 0
 #elif __arc__
 #define R_SYS_ARCH "arc"
 #define R_SYS_BITS R_SYS_BITS_32
+#define R_SYS_ENDIAN 0
 #elif __sparc__
 #define R_SYS_ARCH "sparc"
 #define R_SYS_BITS R_SYS_BITS_32
+#define R_SYS_ENDIAN 1
 #elif __mips__
 #define R_SYS_ARCH "mips"
 #define R_SYS_BITS R_SYS_BITS_32
+#define R_SYS_ENDIAN 1
 #elif __EMSCRIPTEN__
 /* we should default to wasm when ready */
 #define R_SYS_ARCH "x86"
 #define R_SYS_BITS R_SYS_BITS_32
 #else
+#ifdef _MSC_VER
+#ifdef _WIN64
+#define R_SYS_ARCH "x86"
+#define R_SYS_BITS (R_SYS_BITS_32 | R_SYS_BITS_64)
+#define R_SYS_ENDIAN 0
+#define __x86_64__ 1
+#else
+#define R_SYS_ARCH "x86"
+#define R_SYS_BITS (R_SYS_BITS_32)
+#define __i386__ 1
+#define R_SYS_ENDIAN 0
+#endif
+#else
 #define R_SYS_ARCH "unknown"
 #define R_SYS_BITS R_SYS_BITS_32
+#define R_SYS_ENDIAN 0
+#endif
 #endif
 
 #define R_SYS_ENDIAN_NONE 0
@@ -464,11 +520,75 @@ enum {
 #define R_SYS_OS "unknown"
 #endif
 
+#if __GNUC__
+#  define r_sys_trap() __builtin_trap()
+#else
+#  if __i386__ || __x86_64__
+#    define r_sys_trap() __asm__ __volatile__ ("int3")
+#  elif __arm__
+#    define r_sys_trap() __asm__ __volatile__ ("bkpt")
+#  elif __arm64__
+#    define r_sys_trap() __asm__ __volatile__ ("brk #1")
+#  else
+#    define r_sys_trap() __asm__ __volatile__ (".word 0");
+#  endif
+#endif
+
 #ifdef __cplusplus
 }
 #endif
 
-#endif
+static inline void r_run_call1 (void *fcn, void *arg1) {
+	((void (*)(void *))(fcn))(arg1);
+}
+
+static inline void r_run_call2 (void *fcn, void *arg1, void *arg2) {
+	((void (*)(void *, void *))(fcn))(arg1, arg2);
+}
+
+static inline void r_run_call3 (void *fcn, void *arg1, void *arg2, void *arg3) {
+	((void (*)(void *, void *, void *))(fcn))(arg1, arg2, arg3);
+}
+
+static inline void r_run_call4 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4) {
+	((void (*)(void *, void *, void *, void *))(fcn))(arg1, arg2, arg3, arg4);
+}
+
+static inline void r_run_call5 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5) {
+	((void (*)(void *, void *, void *, void *, void *))(fcn))(arg1, arg2, arg3, arg4, arg5);
+}
+
+static inline void r_run_call6 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+	void *arg6) {
+	((void (*)(void *, void *, void *, void *, void *, void *))(fcn))
+		(arg1, arg2, arg3, arg4, arg5, arg6);
+}
+
+static inline void r_run_call7 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+	void *arg6, void *arg7) {
+	((void (*)(void *, void *, void *, void *, void *, void *, void *))(fcn))
+		(arg1, arg2, arg3, arg4, arg5, arg6, arg7);
+}
+
+static inline void r_run_call8 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+	void *arg6, void *arg7, void *arg8) {
+	((void (*)(void *, void *, void *, void *, void *, void *, void *, void *))(fcn))
+		(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
+}
+
+static inline void r_run_call9 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+	void *arg6, void *arg7, void *arg8, void *arg9) {
+	((void (*)(void *, void *, void *, void *, void *, void *, void *, void *, void *))(fcn))
+		(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
+}
+
+static inline void r_run_call10 (void *fcn, void *arg1, void *arg2, void *arg3, void *arg4, void *arg5,
+	void *arg6, void *arg7, void *arg8, void *arg9, void *arg10) {
+	((void (*)(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *))(fcn))
+		(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10);
+}
+
+#endif // R2_TYPES_H
 
 // Usage: R_DEFINE_OBJECT(r_asm);
 #if 0
@@ -480,4 +600,3 @@ enum {
     return (type##_deinit(foo), free(foo), NULL); \
  }
 #endif
-
